/*
    screen.c - User Interface
	Copyright (C) 2009 by H. Raap

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "screen.h"
#include "window.h"
#include "keyboard.h"
#include "screen_playing.h"
#include "screen_tracklist.h"
#include "screen_playlist.h"
#include "mpd.h"

/* We have a global variable which tells us which screen is currently running. 
	We decide which windows will be shown and how we react to key presses depending on this variable.
	The variable is called cur_screen.
*/

static enum SCREEN cur_screen;



/* ------------------------------------------------------------------------------------------------- */

/* The current screen has been exited. Show a new screen */
void
screen_exit(enum SCREEN screen){
	switch (screen){
		case PLAYING_SCREEN:
			playing_screen_exit();
			break;
			
		case PLAYLIST_SCREEN:
			playlist_screen_exit();
			break;
			
		case TRACKLIST_SCREEN:
			tracklist_screen_exit();

		default:
			break;
	};	
}

void
screen_enter(enum SCREEN screen){
	cur_screen = screen;
	switch(screen){
		case PLAYING_SCREEN:
			playing_screen_enter();	
			break;
		case PLAYLIST_SCREEN:
			playlist_screen_enter();
			break;
		case TRACKLIST_SCREEN:
			tracklist_screen_enter();
			break;
		default:
			break;
	};
};
	

/* Initialize the main screen.
	Start with top menu system. 
	We clear the whole screen.
	A task is started which grabs keyboard input and informs controller about user intentions.
*/
void 
mainscreen_init(void) {
	
	init_font();
	set_font(MEDIUMFONT);	
	
	win_scroll_init();
	
	playing_screen_init();
	playlist_screen_init();
	tracklist_screen_init();
		
	screen_enter(PLAYING_SCREEN);
};
	
void switch_screen(enum SCREEN oldscreen, enum SCREEN newscreen){
	screen_exit(oldscreen);
	screen_enter(newscreen);
};


// TODO this just gives a level of indirection for key presses.
//	we should better use a variable with a function to handle this
void 
user_pressed_a_key(int cur_key){
	/* Now this is the same for all screens: */
	switch (cur_key) {
		case KEY_Betty:
			switch_screen(cur_screen, PLAYLIST_SCREEN);
			return;
		default:
			break;
	};
	
	switch (cur_screen) {
		case PLAYING_SCREEN:
			playing_keypress(cur_key);
			break;
			
		case PLAYLIST_SCREEN:
			playlist_keypress(cur_key);
			break;
				
		case TRACKLIST_SCREEN:
			tracklist_keypress(cur_key);
			break;
			
		default:
			break;
	};
};

static int user_req_arg;

/* Check if any screen has a user request */
enum USER_CMD 
user_request(void){
	enum USER_CMD req;
	
	if ( (req = playing_user_request()) != NO_CMD ){
		user_req_arg = playing_arg();
		return (req);
	};
	
	if ( (req = playlist_user_request()) != NO_CMD){
		user_req_arg = playlist_arg();
		return (req);
	};
	
	if ( (req = tracklist_user_request()) != NO_CMD){
		user_req_arg = tracklist_arg();
		return (req);
	};
	return NO_CMD;
};

int
user_request_arg(){
	return user_req_arg;
};

