/*
    window.h - window handling and drawing
	Copyright (C) 2009 by H. Raap

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WINDOW_H
#define WINDOW_H

#include "global.h"
#include "kernel.h"
#include "fonty.h"
#include "lcd.h"


/* A text window is a rectangular area on screen.
	Each window has a text buffer which it displays.
	It can do either automatic scrolling or line wrapping and/or clipping.
	It can display a border around itself.
*/

struct Window {
	uint8 start_row, start_col;
	uint8 width, height;
	uint8 fg_color;
	uint8 bg_color;
	uint8 border;
	uint8 cur_char;
	uint8 cur_col;
	uint8 txt_offs_row;
	uint8 txt_offs_col;
	int	 text_len;			// == strlen(txt)
	enum FONT font;
	uint8 fits;
	uint8 scroll;
	uint8 flags;
	uint8 buffer_lim;
	char *txt;
};

/* Start of text 
	TODO think about including border here
*/
#define win_txt_row(win) ( (win)->start_row + (win)->txt_offs_row )
#define win_txt_col(win) ( (win)->start_col + (win)->txt_offs_col )
#define win_txt_width(win) ( (win)->width - 2*(win)->txt_offs_col ) 
#define win_txt_height(win) ( (win)->height - 2*(win)->txt_offs_row )

/* Last column in window + 1 for text */
#define txt_col_lim(win) ((win)->start_col + (win)->width - (win)->txt_offs_col)

/* The flags variable in struct Window has these semantics: */

/* Is the window visible */
#define	WINFLG_VISIBLE	(1<<0)
/* Shall the content be scrolled when necessary */
#define WINFLG_SCROLL	(1<<1)	
/* Shall text be centered in the window */
#define WINFLG_CENTER	(1<<2)
/* Is this a text window */
#define WINFLG_TEXT		(1<<3)
/* Is this a ramp */
#define WINFLG_RAMP		(1<<4)
/* Is this a progress bar */
#define WINFLG_BAR		(1<<5)


void win_changed(int pos, char *s);
void view_title_changed (char *s);
void view_artist_changed (char *s);
void view_volume_changed (int vol);
void view_time_changed(int te, int tt);
void playing_keypress(int cur_key);
void win_new_text(struct Window *win, char *s);
void win_init(struct Window *win, uint8 row, uint8 col, uint8 height, uint8 width, uint8 border, char *txt);
void win_draw_border(struct Window *win);
void win_redraw(struct Window *win);
void win_scroll(struct Window *win);
void win_unscroll(struct Window *win);
void win_scroll_init();
void win_clear(struct Window *win, int clr_border);

#endif

