/* model.h 
	Copyright (C) 2009 by H. Raap

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MODEL_H
#define MODEL_H

/* mpd can be in one of those states TODO maybe add a state ERROR */
enum PLAYSTATE {UNKNOWN, STOP, PAUSE, PLAY};


/* MPD gives a 0-based answer on song. So we have to define 2 values, -1 for unknown and -2 for no current song */
#define NO_SONG			-2
#define SONG_UNKNOWN	-1

#define VOLUME_CHANGED		(1<<0)
#define TITLE_CHANGED		(1<<1)
#define ARTIST_CHANGED		(1<<2)
#define TIME_CHANGED		(1<<3)
#define STATE_CHANGED		(1<<4)
#define PL_NAMES_CHANGED	(1<<5)
#define TRACKLIST_CHANGED	(1<<6)
#define POS_CHANGED			(1<<7)

/* Maximum length of a tracklist entry including trailing 0 
	We expect to have a lot of those entries, so we keep them small
*/
#define TL_ENTRY_LENGTH 64

void mpd_inc_time();
enum USER_CMD model_needs_action(int *arg);
int model_get_changed(void);
void start_reading_playlists(void);
void user_toggle_pause(void);
void user_set_state(enum PLAYSTATE s);
void mpd_want_song(int p);
void mpd_want_song_add(int offs);
void mpd_want_time_add(int offs);
void user_mute(void);
void mpd_want_volume_add(int chg);
void mpd_want_playlist(int idx);
void error_inc(int amount);
void error_dec(int amount);
int get_comm_error();
void model_reset_changed();
void mpd_set_time(int elapsed, int total);
void user_song_unknown();
void tl_store_track(char *title, char *artist, int track_no);
void tracklist_range_set(int start, int end);
int mpd_get_time_total();
int mpd_get_time_elapsed();
int user_get_state();
int mpd_get_state();
char *mpd_get_curtitle();
char *mpd_get_curartist();
int	user_get_pos();
int	mpd_get_volume();
void mpd_set_newplaylist();
void mpd_set_title(char *s);
void mpd_set_artist(char *s);
void mpd_set_pos(int newpos);
void set_playlistlength(int n);
void mpd_set_state(enum PLAYSTATE newstate);
void mpd_set_volume(int vol);
void mpd_set_playlistname(char *s);
char *pl_get_name(int idx);




#endif

